from aio_client.provider.api import PostProviderRequest
from aio_client.provider.api import push_request

from concentrator.smev3_v321.base.constants import SUCCESS_MESSAGE
from concentrator.smev3_v321.base.utils import BaseExecutor
from concentrator.smev3_v321.base.utils import SMEV3Response
from concentrator.smev3_v321.base.utils import render_type2xml
from concentrator.smev3_v321.service_types import kinder_conc

from .application import ApplicationManager


class ApplicationRequestExecutor(BaseExecutor):
    """Исполнитель сервиса ApplicationRequest."""

    name_service = 'ApplicationRequest'
    type_service = kinder_conc.ApplicationType

    @classmethod
    def process(cls, message, request_body, **kwargs):

        attachments = message['attachments']

        response_body = render_type2xml(
            cls.get_response(
                request_body.ApplicationRequest, attachments, message, **kwargs
            ),
            name_type='FormDataResponse')

        response = push_request(PostProviderRequest(
            origin_message_id=message['origin_message_id'],
            body=response_body,
            message_type=message['message_type'],
            replay_to=message['replay_to'],
            is_test_message=message['is_test_message']))

        return SMEV3Response(
            response,
            {
                'method_name': cls.name_service,
                'response': response_body,
                'result': SUCCESS_MESSAGE
            })

    @classmethod
    def get_response(cls, application_request, attachments, message, **kwargs):
        """Формирует и возвращает ответ.

        :param application_request: Данные заявления
        :type application_request: ApplicationType
        :param attachments: Прикрепленные файлы (формат JSON)
        :type attachments: Union[list, str, bytearray, bytes]
        :param message: Параметры запроса
        :type message: dict

        :return: Ответ
        :rtype: FormDataResponseType
        """

        # Если параметр установлен, то при валидации Желаемой даты зачисления,
        # она будет сравниваться с датой из тэга FilingDate
        compare_date_with_filing = kwargs.get('compare_with_filing', False)

        result = ApplicationManager(
            application_request, attachments, message, compare_date_with_filing
        ).run()

        response = kinder_conc.FormDataResponseType(
            changeOrderInfo=kinder_conc.changeOrderInfoType(
                orderId=kinder_conc.orderIdType(result.order_id),
                statusCode=kinder_conc.statusCodeType(result.org_code),
                comment=result.comment))

        return response
